Percent = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0")

function Percent:OnInitialize()
	local targetFrame = CreateFrame("Frame", "PercentTargetFrame", TargetFrame)
	targetFrame:SetFrameStrata("HIGH")
	
	targetHealthText = targetFrame:CreateFontString("PercentTargetHealthText")
    targetHealthText:SetFontObject(GameFontNormalSmall)
    targetHealthText:SetHeight(32)
    targetHealthText:SetPoint("TOP", TargetFrameHealthBar, "TOP", 0, 10)
    targetHealthText:SetTextColor(1, 1, 1, 1)
	
	targetPowerText = targetFrame:CreateFontString("PercentTargetPowerText")
    targetPowerText:SetFontObject(GameFontNormalSmall)
    targetPowerText:SetHeight(32)
    targetPowerText:SetPoint("TOP", TargetFrameManaBar, "TOP", 0, 10)
    targetPowerText:SetTextColor(1, 1, 1, 1)
	
	local playerFrame = CreateFrame("Frame", "PercentPlayerFrame", PlayerFrame)
	playerFrame:SetFrameStrata("HIGH")
	
	-- Status bar
	playerHealthText = playerFrame:CreateFontString("PercentPlayerHealthText")
    playerHealthText:SetFontObject(GameFontNormalSmall)
    playerHealthText:SetHeight(32)
    playerHealthText:SetPoint("TOP", PlayerFrameHealthBar, "TOP", 0, 10)
    playerHealthText:SetTextColor(1, 1, 1, 1)
	
	playerPowerText = playerFrame:CreateFontString("PercentPlayerPowerText")
    playerPowerText:SetFontObject(GameFontNormalSmall)
    playerPowerText:SetHeight(32)
    playerPowerText:SetPoint("TOP", PlayerFrameManaBar, "TOP", 0, 10)
    playerPowerText:SetTextColor(1, 1, 1, 1)
	
	-- Right of the status bar
	playerHealthPercentText = playerFrame:CreateFontString("PercentPlayerHealthPercentText")
    playerHealthPercentText:SetFontObject(GameFontNormalSmall)
    playerHealthPercentText:SetHeight(32)
    playerHealthPercentText:SetPoint("LEFT", PlayerFrameHealthBar, "RIGHT", 5, 0)
    playerHealthPercentText:SetTextColor(1, 1, 1, 1)
	
	playerPowerPercentText = playerFrame:CreateFontString("PercentPlayerPowerPercentText")
    playerPowerPercentText:SetFontObject(GameFontNormalSmall)
    playerPowerPercentText:SetHeight(32)
    playerPowerPercentText:SetPoint("LEFT", PlayerFrameManaBar, "RIGHT", 5, 0)
    playerPowerPercentText:SetTextColor(1, 1, 1, 1)
	
	-- Disables UI Status bar text
	PlayerFrameHealthBar:SetScript("OnUpdate", function()
		if PlayerFrameHealthBarText:IsVisible() then
			PlayerFrameHealthBarText:Hide()
		end
	end)
	PlayerFrameManaBar:SetScript("OnUpdate", function()
		if PlayerFrameManaBarText:IsVisible() then
			PlayerFrameManaBarText:Hide()
		end
	end)
end

function Percent:OnEnable()
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UNIT_HEALTH")
	self:RegisterEvent("UNIT_MANA")
	self:RegisterEvent("UNIT_RAGE")
	self:RegisterEvent("UNIT_ENERGY")
	self:RegisterEvent("UNIT_DISPLAYPOWER")
	
	-- Shows percent that you always see
	self:UpdateHealth("player")
	self:UpdatePower("player")
	self:UpdateHealthPercent("player")
	self:UpdatePowerPercent("player")
end

function Percent:PLAYER_TARGET_CHANGED()
	if UnitExists("target") then
		self:UpdateHealth("target")
		self:UpdatePower("target")
		if not targetHealthText:IsVisible() then targetHealthText:Show() end
		if not targetPowerText:IsVisible() then targetPowerText:Show() end
	else
		if targetHealthText:IsVisible() then targetHealthText:Hide() end
		if targetPowerText:IsVisible() then targetPowerText:Hide() end
	end
end

function Percent:UNIT_HEALTH()
	if (arg1 == "target") or (arg1 == "player") then
		self:UpdateHealth(arg1)
	end
	
	if arg1 == "player" then
		self:UpdateHealthPercent(arg1)
	end
end

function Percent:UNIT_MANA()
	if (arg1 == "target") or (arg1 == "player") then
		self:UpdatePower(arg1);
	end
	
	if arg1 == "player" then
		self:UpdatePowerPercent(arg1)
	end
end

function Percent:UNIT_RAGE()
	if (arg1 == "target") or (arg1 == "player") then
		self:UpdatePower(arg1);
	end
	
	if arg1 == "player" then
		self:UpdatePowerPercent(arg1)
	end
end

function Percent:UNIT_ENERGY()
	if (arg1 == "target") or (arg1 == "player") then
		self:UpdatePower(arg1);
	end
	
	if arg1 == "player" then
		self:UpdatePowerPercent(arg1)
	end
end

function Percent:UNIT_DISPLAYPOWER()
	if (arg1 == "target") or (arg1 == "player") then
		self:UpdatePower(arg1);
	end
	
	if arg1 == "player" then
		self:UpdatePowerPercent(arg1)
	end
end

function Percent:UpdateHealth(unit)
	fontstring = getglobal(unit .. "HealthText")
	fontstring:SetText(self:GetUnitHealthValue(unit))
end

function Percent:UpdatePower(unit)
	fontstring = getglobal(unit .. "PowerText")
	fontstring:SetText(self:GetUnitPowerValue(unit))
end

function Percent:UpdateHealthPercent(unit)
	fontstring = getglobal(unit .. "HealthPercentText")
	fontstring:SetText(self:GetUnitHealthPercent(unit))
end

function Percent:UpdatePowerPercent(unit)
	fontstring = getglobal(unit .. "PowerPercentText")
	fontstring:SetText(self:GetUnitPowerPercent(unit))
end

function Percent:GetUnitHealthValue(unit)
	if UnitHealth(unit) == 0 then
		return ""
	end
	
	if UnitIsUnit("player", unit) or UnitInParty(unit) or UnitInRaid(unit) then
		return UnitHealth(unit) .. "/" .. UnitHealthMax(unit)
	end
	
	return math.ceil((UnitHealth(unit) / UnitHealthMax(unit)) * 100) .. "%"
end

function Percent:GetUnitPowerValue(unit)
	if UnitManaMax(unit) == 0 then
		return ""
	end
	
	return UnitMana(unit) .. "/" .. UnitManaMax(unit)
end

function Percent:GetUnitHealthPercent(unit)
	if UnitHealth(unit) == 0 then
		return ""
	end
	
	return math.ceil((UnitHealth(unit) / UnitHealthMax(unit)) * 100) .. "%"
end

function Percent:GetUnitPowerPercent(unit)
	if UnitManaMax(unit) == 0 then
		return ""
	end
	
	return math.ceil((UnitMana(unit) / UnitManaMax(unit)) * 100) .. "%"
end